<?php

class Pyschometericchart {
	

	public function findpressure($altitude) {
		
		//if (($altitude<-5000)||($altitude>11000)) echo("Altitude accurate from -5,000 to 11,000m");
		$pressure = 101325*pow(1 - 2.25577e-5*$altitude,5.2559);
		return $pressure;
	}

	public function saturation_pressure($t) {
	
		$c1 = -5.6745359e3;
		$c2 = 6.3925247e0;
		$c3 = -9.677843e-3;
		$c4 = 6.2215701e-7;
		$c5 = 2.0747825e-9;
		$c6 = -9.484024e-13;
		$c7 = 4.1635019e0;

		$c8=-5.8002206e3;
		$c9=1.3914993;
		$c10=-4.8640239e-2;
		$c11=4.1764768e-5;
		$c12=-1.4452093e-8;
		$c13=6.5459673;

		if ($t>0)
		  {
			$t=$t+273.15;
			$pressure=exp($c8/$t+$c9+$c10*$t+$c11*$t*$t+$c12*$t*$t*$t+$c13*log($t));
		  } else { 
			$t=$t+273.15;
			$pressure=exp($c1/$t+$c2+$c3*$t+$c4*$t*$t+$c5*$t*$t*$t+$c6*$t*$t*$t*$t+$c7*log($t)); 
		}    
		
		return $pressure;
	}
	
	public function humidity_ratio($p_w,$p) {			
		return 0.62198*$p_w/($p-$p_w);
	}

	public function equation_35($t_wb,$ws_wb,$t) {
		return (((2501-2.381*$t_wb)*$ws_wb - 1.006*($t-$t_wb))/(2501+1.805*$t-4.186*$t_wb));
	}


	public function degree_saturation($w,$ws) {
		return $w/$ws;
	}

	public function rel_humidity($mew,$p_ws,$p) {
		$thi=$mew/(1.0-(1.0-$mew)*($p_ws/$p));
		return $thi*100;
	}

	public function specific_volume($t,$w,$p) {
		$v = 287.1*($t+273.15)*(1+1.6078*$w)/$p;
		//converted for pressure in Pa not kPa
		return $v;
	}

	public function enthalpy($w,$t) {
		$h=1.006*$t + $w*(2501+1.805*$t);
		return $h;
	}


	public function equation_36($p,$w) {
		$pressure=$p*$w/(0.62198+$w);
		return $pressure;
	}

	public function dewpoint($p_w) {
		$p_w=$p_w/1000.0;
		$a = log($p_w);
		$c14=6.54;
		$c15=14.526;
		$c16=0.7389;
		$c17=0.09486;
		$c18=0.4569;

		$dew= $c14+$c15*$a+$c16*$a*$a+$c17*$a*$a*$a+$c18*pow($p_w,0.1984);
		if ($dew < 0) {
			$dew = 6.09 + 12.608*$a + 0.4959*$a*$a;
		}

		return $dew;
	}

	public function density($v,$w) {
		$dens=(1.0/$v)*(1.0+$w);
		return $dens;
	}

	public function calcwetbulb($t,$p,$w) {
		$t_wb=$t;
		$count=1;
		$error=1.0;

		while (($count < 10000)&&($error > 0.001)) {
		   $p_ws_wb=PyschometericChart::saturation_pressure($t_wb);
		   $ws_wb=PyschometericChart::humidity_ratio($p_ws_wb,$p);
		   $test=(2501*($ws_wb-$w)-$t*(1.006+1.805*$w))/(2.381*$ws_wb-1.006-4.186*$w);
		   $error = $t_wb-$test;
		   $t_wb = $t_wb - $error/100;
		   $count = $count+1;
		}
		
		if ($count > 9999) echo "calculation error in wet bulb temperature";
		return $t_wb;
	}
	
	public function calc($t,$rh,$altitude,$t_wb,$t_d) {  
       
		// Wet Bulb Temprature is set 
		if ($t_wb) {
			if ($t_wb>$t) {
				die("Wet bulb cannot be greater than Dry bulb!");
			}
			
			$p = $this->findpressure($altitude);
			$p_ws_wb = $this->saturation_pressure($t_wb);
			$ws_wb = $this->humidity_ratio($p_ws_wb,$p);
			$w = $this->equation_35($t_wb,$ws_wb,$t);
			$p_ws = $this->saturation_pressure($t);
			$ws = $this->humidity_ratio($p_ws,$p);
			$mew = $this->degree_saturation($w,$ws); 
			$rh = $this->rel_humidity($mew,$p_ws,$p);
			$v = $this->specific_volume($t,$w,$p);
			$h = $this->enthalpy($w,$t);
			$p_w = $this->equation_36($p,$w);
			$t_d = $this->dewpoint($p_w);
			$rho = $this->density($v,$w);  
			
		} else if (($rh>=0) && ($rh <=100)) {
		
			 $p = $this->findpressure($altitude);
			 $p_ws = $this->saturation_pressure($t);
			 $p_w = $p_ws*$rh/100.0;
			 $w = $this->humidity_ratio($p_w,$p);
			 $ws = $this->humidity_ratio($p_ws,$p);
			 $mew = $this->degree_saturation($w,$ws); 
			 $v = $this->specific_volume($t,$w,$p);
			 $h = $this->enthalpy($w,$t);
			 $t_d = $this->dewpoint($p_w);
			 $rho = $this->density($v,$w);  
			 $t_wb = $this->calcwetbulb($t,$p,$w);
			 
		} else if ($t_d) {
		
			 $p = $this->findpressure($altitude);
			 $p_w = $this->saturation_pressure($t_d);
			 $w = $this->humidity_ratio($p_w,$p);
			 $p_ws = $this->saturation_pressure($t);     
			 $ws = $this->humidity_ratio($p_ws,$p);     
			 $mew = $this->degree_saturation($w,$ws); 
			 $rh = $this->rel_humidity($mew,$p_ws,$p);
			 $v = $this->specific_volume($t,$w,$p);
			 $h = $this->enthalpy($w,$t);
			 $rho = $this->density($v,$w);  
			 $t_wb = $this->calcwetbulb($t,$p,$w);
			 
		} else {
			die("You must enter either Wet Bulb, Relative Humdity or Dewpoint temperature");
		}    

		//if ($t>80 || $t<-60) echo("Results may be inaccurate for temperatures > 80 or < -60");

		$values = [
					"t"=>$t,
					"t_wb"=>$t_wb,
					"rh"=>$rh,
					"t_d"=>$t_d,
					"altitude"=>$altitude,
					"enthalpy"=>$h,
					"density"=>$rho,
					"hum_ratio"=>$w,
					"pressure"=>$p,
					"spec_volume"=>$v,
					"p_w"=>$p_w,
					"p_s"=>$p_ws
				];

		return $values;
	}

}
?>